'use strict';

const getAllSubCombosHelper = ({
  substr,
  buffer,
  limit,
  trieRoot
}) => {
  const finalPasswords = [];
  // eslint-disable-next-line max-statements
  const helper = (index, changes) => {
    if (finalPasswords.length >= limit) {
      return;
    }
    if (index === substr.length) {
      finalPasswords.push({
        password: buffer.join(''),
        changes
      });
      return;
    }
    const firstChar = substr.charAt(index);
    // first, generate all combos without doing a substitution at this index
    buffer.push(firstChar);
    helper(index + 1, changes);
    buffer.pop();
    // next, exhaust all possible substitutions at this index
    let cur = trieRoot;
    for (let i = index; i < substr.length; i += 1) {
      const character = substr.charAt(i);
      cur = cur.getChild(character);
      if (!cur) {
        return;
      }
      if (cur.isTerminal()) {
        const subs = cur.subs;
        // eslint-disable-next-line no-restricted-syntax
        for (const sub of subs) {
          buffer.push(sub);
          const newSubs = changes.concat({
            letter: sub,
            substitution: cur.parents.join('')
          });
          // recursively build the rest of the string
          helper(i + 1, newSubs);
          // backtrack by ignoring the added postfix
          buffer.splice(-sub.length);
          if (finalPasswords.length >= limit) {
            return;
          }
        }
      }
    }
  };
  helper(0, []);
  return finalPasswords;
};
const getCleanPasswords = (string, limit, trieRoot) => {
  return getAllSubCombosHelper({
    substr: string,
    buffer: [],
    limit,
    trieRoot
  });
};

module.exports = getCleanPasswords;
//# sourceMappingURL=getCleanPasswords.js.map
